from typing import List, Optional
from datetime import datetime
from dateutil import parser
from .stage import Stage
from .team import Team


class Contact:
    type: str
    details: str

    def __init__(self, type: str, details: str) -> None:
        self.type = type
        self.details = details


class TournamentCustomField:
    name: str
    public: bool
    id: str

    def __init__(self, data: dict) -> None:
        self.name = data.get("name")
        self.public = data.get("public")
        self.id = data.get("_id")


class EmailsSent:
    one_day: bool
    now: bool
    pending_team_notification: bool

    def __init__(self, data: dict) -> None:
        self.one_day = data.get("oneDay")
        self.now = data.get("now")
        self.pending_team_notification = data.get("pendingTeamNotification")


class Features:
    enable_valorant_asr: bool

    def __init__(self, data: dict) -> None:
        self.enable_valorant_asr = data.get("enableValorantASR")


class Rules:
    complete: str
    critical: str

    def __init__(self, complete: str, critical: str) -> None:
        self.complete = complete
        self.critical = critical


class Tournament:
    id: str
    start_time: Optional[datetime]
    rules: Rules
    players_per_team: int
    custom_fields: List[TournamentCustomField]
    user_can_report: bool
    name: str
    about: str
    banner_url: str
    contact: Contact
    contact_details: str
    prizes: str
    schedule: str
    features: Features
    check_in_required: bool
    check_in_starts: int
    type: str
    has_max_players: bool
    has_registration_cap: bool
    team_cap: int
    max_players: int
    country_flags_on_brackets: bool
    has_points: bool
    is_published: bool
    service_fee_percent: int
    game_name: str
    is_featured: bool
    is_public: bool
    is_suspended: bool
    is_roster_locked: bool
    created_at: Optional[datetime]
    updated_at: Optional[datetime]
    check_in_start_time: Optional[datetime]
    slug: str
    cloned_from_tournament_id: str
    registration_enabled: bool
    emails_sent: EmailsSent
    organization_id: str
    game_id: str
    stage_ids: List[str]
    last_completed_match_at: Optional[datetime]
    template_id: str

    teams: List[Team]
    stages: List[Stage]

    def __init__(self, data: dict):
        self.id = data.get("_id")
        if "startTime" in data:
            self.start_time = parser.isoparse(data.get("startTime"))
        self.rules = Rules(**data.get("rules"))
        self.players_per_team = data.get("playersPerTeam")
        self.custom_fields = [TournamentCustomField(x) for x in data.get("customFields")]
        self.user_can_report = data.get("userCanReport")
        self.name = data.get("name")
        self.about = data.get("about")
        self.banner_url = data.get("bannerUrl")
        self.contact = Contact(**data.get("contact"))
        self.contact_details = data.get("contactDetails")
        self.prizes = data.get("prizes")
        self.schedule = data.get("schedule")
        self.features = Features(data.get("features", {}))
        self.check_in_required = data.get("checkInRequired")
        self.check_in_starts = data.get("checkInStarts")
        self.type = data.get("type")
        self.has_max_players = data.get("hasMaxPlayers")
        self.has_registration_cap = data.get("hasRegistrationCap")
        self.team_cap = data.get("teamCap")
        self.max_players = data.get("maxPlayers")
        self.country_flags_on_brackets = data.get("countryFlagsOnBrackets")
        self.has_points = data.get("hasPoints")
        self.is_published = data.get("isPublished")
        self.service_fee_percent = data.get("serviceFeePercent")
        self.game_name = data.get("gameName")
        self.is_featured = data.get("isFeatured")
        self.is_public = data.get("isPublic")
        self.is_suspended = data.get("isSuspended")
        self.is_roster_locked = data.get("isRosterLocked")
        if "createdAt" in data:
            self.created_at = parser.isoparse(data.get("createdAt"))
        if "updatedAt" in data:
            self.updated_at = parser.isoparse(data.get("updatedAt"))
        if "checkInStartTime" in data:
            self.check_in_start_time = parser.isoparse(data.get("checkInStartTime"))
        self.slug = data.get("slug")
        self.cloned_from_tournament_id = data.get("clonedFromTournamentID")
        self.registration_enabled = data.get("registrationEnabled")
        self.emails_sent = EmailsSent(data.get("emailsSent", {}))
        self.organization_id = data.get("organizationID")
        self.game_id = data.get("gameID")
        self.stage_ids = data.get("stageIDs")
        if "lastCompletedMatchAt" in data:
            self.last_completed_match_at = parser.isoparse(data.get("lastCompletedMatchAt"))
        self.template_id = data.get("templateID")
        stages_data = data.get("stages", [])
        self.stages = []
        for stage_data in stages_data:
            self.stages.append(Stage(stage_data))
        self.teams = []

    def add_stage(self, stage: Stage):
        self.stages.append(stage)

    def add_teams(self, data: List[dict]):
        for team_data in data:
            self.teams.append(Team(team_data))

    def get_team_from_team_id(self, team_id: str) -> Team:
        for team in self.teams:
            if team.id == team_id:
                return team
        return None

    def get_team_from_persistent_team_id(self, persistent_team_id: str) -> Team:
        for team in self.teams:
            if team.persistent_team_id == persistent_team_id:
                return team
        return None

    def get_player_from_persistent_player_id(self, persistent_player_id: str):
        for team in self.teams:
            for player in team.players:
                if player.persistent_player_id == persistent_player_id:
                    return player
        return None







from .tournament import Tournament
from .tournament.stage import Stage
from .tournament.team import Team, Player
from .tournament.match import Match
from .tournament.standing import Standing, LadderStanding, SwissStanding, BracketStanding, RoundRobinStanding
from .organization import Organization, organization_from_dict, SearchTournament, tournaments_from_org_search



from typing import Optional, List, Any, Optional
from datetime import datetime
from dateutil import parser


class Player:
    id: str
    on_team: bool
    is_free_agent: bool
    be_captain: bool
    in_game_name: str
    persistent_player_id: Optional[str]
    user_id: Optional[str]
    owner_id: str
    created_at: Optional[datetime]
    updated_at: Optional[datetime]
    user_slug: Optional[str]
    username: Optional[str]
    avatar_url: Optional[str]
    discord_id: Optional[str]

    def __init__(self, data: dict) -> None:
        self.id = data.get("_id")
        self.on_team = data.get("onTeam")
        self.is_free_agent = data.get("isFreeAgent")
        self.be_captain = data.get("beCaptain")
        self.in_game_name = data.get("inGameName")
        self.persistent_player_id = data.get("persistentPlayerID")
        self.user_id = data.get("userID")
        self.owner_id = data.get("ownerID")
        if "createdAt" in data:
            self.created_at = parser.isoparse(data.get("createdAt"))
        if "updatedAt" in data:
            self.updated_at = parser.isoparse(data.get("updatedAt"))
        self.user_slug = data.get("userSlug")
        self.username = data.get("username")
        self.avatar_url = data.get("avatarUrl")
        self.discord_id = None

    def __repr__(self) -> str:
        return f"<Player id={self.id} in_game_name={self.in_game_name}>"


class CustomField:
    id: str
    value: str

    def __init__(self, data: dict) -> None:
        self.id = data.get("_id")
        self.value = data.get("value")


class PersistentTeam:
    id: str
    name: str
    logo_url: str
    short_description: str
    banner_url: str
    sponsor_banner_url: str
    created_at: Optional[datetime]
    updated_at: Optional[datetime]
    persistent_player_ids: List[str]
    persistent_captain_id: str
    pending_player_ids: List[Any]
    owner_id: str
    deleted_at: Optional[datetime]

    def __init__(self, data: dict) -> None:
        self.id = data.get("_id")
        self.name = data.get("name")
        self.logo_url = data.get("logoUrl")
        self.short_description = data.get("shortDescription")
        self.banner_url = data.get("bannerUrl")
        self.sponsor_banner_url = data.get("sponsorBannerUrl")
        if "createdAt" in data:
            self.created_at = parser.isoparse(data.get("createdAt"))
        if "updatedAt" in data:
            self.updated_at = parser.isoparse(data.get("updatedAt"))
        self.persistent_player_ids = data.get("persistentPlayerIDs")
        self.persistent_captain_id = data.get("persistentCaptainID")
        self.pending_player_ids = data.get("pendingPlayerIDs")
        self.owner_id = data.get("ownerID")
        if data.get("deletedAt"):
            self.deleted_at = parser.isoparse(data.get("deletedAt"))


class Team:
    id: str
    name: str
    pending_team_id: str
    persistent_team_id: str
    tournament_id: str
    user_id: str
    custom_fields: List[CustomField]
    owner_id: str
    created_at: Optional[datetime]
    player_ids: List[str]
    captain_id: str
    checked_in_at: Optional[datetime]
    captain: Player
    players: List[Player]
    persistent_team: PersistentTeam

    def __init__(self, data: dict) -> None:
        self.id = data.get("_id")
        self.name = data.get("name")
        self.pending_team_id = data.get("pendingTeamID")
        self.persistent_team_id = data.get("persistentTeamID")
        self.tournament_id = data.get("tournamentID")
        self.user_id = data.get("userID")
        self.custom_fields = [CustomField(c) for c in data.get("customFields", [])]
        self.owner_id = data.get("ownerID")
        if "createdAt" in data:
            self.created_at = parser.isoparse(data.get("createdAt"))
        self.player_ids = data.get("playerIDs")
        self.captain_id = data.get("captainID")
        if "checkedInAt" in data:
            self.checked_in_at = parser.isoparse(data.get("checkedInAt"))
        self.captain = Player(data.get("captain"))
        self.players = [Player(p) for p in data.get("players", [])]
        self.persistent_team = PersistentTeam(data.get("persistentTeam", {}))

    def __repr__(self) -> str:
        return f"<Team id={self.id} name={self.name}>"



import asyncio
import aiohttp
from typing import Optional, List
from .models import Organization, organization_from_dict, SearchTournament, tournaments_from_org_search


class OrganizationClient:
    def __init__(self, headers: dict = None):
        self.headers = headers or {}

    async def get_org(self, org_slug: str) -> Optional[Organization]:
        """
        Get data on a Battlefy Org
        :param org_slug:
        :return: Organisation
        """
        async with aiohttp.ClientSession() as session:
            async with session.get(
                    f'https://dtmwra1jsgyb0.cloudfront.net/organizations?slug={org_slug}&extend%5Badmins%5D=true&extend%5Bmoderators%5D=true&extend%5Bowner%5D=true&extend%5Brunners%5D=true',
                    headers=self.headers) as resp:
                if resp.status == 200:
                    data = await resp.json()
                    orgs = organization_from_dict(data)
                    if len(orgs) > 0:
                        return orgs[0]
                else:
                    return None

    async def __get_page_data(self, org_id: str, page_no: int, tour_list: list):
        async with aiohttp.ClientSession() as session:
            async with session.get(
                    f"https://search.battlefy.com/tournament/organization/{org_id}/past?page={page_no}&size=10",
                    headers=self.headers) as resp:
                if resp.status == 200:
                    page_data = await resp.json()
                    if "tournaments" in page_data:
                        tour_list += tournaments_from_org_search(page_data["tournaments"])
        return tour_list

    async def search_orgs(self, org_id: str) -> List[SearchTournament]:
        tournaments = []
        pages = 0
        async with aiohttp.ClientSession() as session:
            async with session.get(f"https://search.battlefy.com/tournament/organization/{org_id}/past?page=1&size=10"
                    ,headers=self.headers) as resp:
                if resp.status == 200:
                    page_one_data = await resp.json()
                    if "total" in page_one_data:
                        pages = int(page_one_data["total"] / 10)
                        if pages > 0:
                            pages = pages + 1
                        if (page_one_data["total"] % 10) != 0:
                            pages = pages + 1
                    if "tournaments" in page_one_data:
                        tournaments += tournaments_from_org_search(page_one_data["tournaments"])
                else:
                    return []
        async with asyncio.TaskGroup() as g:
            for x in range(2, pages):
                g.create_task(self.__get_page_data(org_id, x, tournaments))
        return tournaments



from dataclasses import dataclass, InitVar, field
from typing import Union
from .team import Team


@dataclass
class Standing:
    standing_data: InitVar[dict]
    team: Team
    set_wins: int = field(init=False, default=0)
    set_losses: int = field(init=False, default=0)
    set_ties: int = field(init=False, default=0)

    def __post_init__(self, standing_data: dict):
        self.set_wins = standing_data.get("wins", 0)
        self.set_losses = standing_data.get("losses", 0)
        self.set_ties = standing_data.get("ties", 0)


@dataclass
class LadderStanding(Standing):
    points: int = field(init=False, default=0)
    game_win_percentage: float = field(init=False, default=0.0)
    match_win_percentage: float = field(init=False, default=0.0)
    disqualified: bool = field(init=False, default=False)
    place: Union[int, None]

    def __post_init__(self, standing_data: dict):
        super().__post_init__(standing_data)
        self.points = standing_data.get("points", 0)
        self.match_win_percentage = standing_data.get("matchWinPercentage", 0)
        self.game_win_percentage = standing_data.get("gameWinPercentage", 0)


@dataclass
class SwissStanding(Standing):
    points: int = field(init=False, default=0)
    opponents_win_percentage: float = field(init=False, default=0.0)
    game_win_percentage: float = field(init=False, default=0.0)
    disqualified: bool = field(init=False, default=False)
    place: Union[int, None]

    def __post_init__(self, standing_data: dict):
        super().__post_init__(standing_data)
        self.points = standing_data.get("points", 0)
        self.opponents_win_percentage = standing_data.get("opponentsMatchWinPercentage", 0)
        self.game_win_percentage = standing_data.get("gameWinPercentage", 0)
        self.disqualified = standing_data.get("disqualified", False)
        if self.disqualified:
            self.place = None


@dataclass
class BracketStanding(Standing):
    place: int = field(init=False, default=0)

    def __post_init__(self, standing_data: dict):
        super().__post_init__(standing_data)
        self.place = standing_data.get("place", 0)

    def set_place(self, place: int):
        self.place = place

    def __repr__(self):
        return f"{self.team.name} - {self.place}"


@dataclass
class RoundRobinStanding(Standing):
    points: int = field(init=False, default=0)
    # Tiebreakers
    match_win_amongst_tied: int = field(init=False, default=0)  # rrt2
    game_win_lose_difference: int = field(init=False, default=0)  # rrt3
    total_win_lose_difference: int = field(init=False, default=0)  # rrt4

    place: Union[int, None] = field(init=False, default=0)

    def __post_init__(self, standing_data: dict):
        super().__post_init__(standing_data)
        self.points = standing_data.get("points", 0)
        self.match_win_amongst_tied = standing_data.get("rrt2", 0)
        self.game_win_lose_difference = standing_data.get("rrt3", 0)
        self.total_win_lose_difference = standing_data.get("rrt4", 0)

